import functools
import asyncio

import msgpack


def make_msgpack_protocol_factory(client_connected_cb, unpacker_args={}, packer_args={}, loop=None):
    if loop is None:
        loop = asyncio.get_event_loop()

    return functools.partial(MsgpackProtocol, client_connected_cb, unpacker_args, packer_args, loop=loop)


class MsgpackProtocol (asyncio.Protocol):
    def __init__(self, client_connected_cb, unpacker_args={}, packer_args={}, loop=None):
        self._client_connected_cb = client_connected_cb
        self._unpacker = msgpack.Unpacker(**unpacker_args)
        self._packer = msgpack.Packer(**packer_args)
        self._more_data_event = asyncio.Event()
        self._eof = False
        self._exc = None
        self._transport = None
        self._loop = loop

    def connection_made(self, transport):
        self._transport = transport
        self._loop.create_task(self._client_connected_cb(self))

    def connection_lost(self, exc):
        self._exc = exc

    def data_received(self, data):
        self._unpacker.feed(data)
        self._more_data_event.set()

    def eof_received(self):
        self._eof = True
        self._more_data_event.set()

    def __aiter__(self):
        return self

    async def __anext__(self):
        while True:
            try:
                return self._unpacker.unpack()
            except msgpack.OutOfData:
                pass
            if self._exc:
                raise self._exc
            if self._eof:
                raise StopAsyncIteration()
            self._more_data_event.clear()
            await self._more_data_event.wait()

    async def read(self):
        try:
            return await self.__anext__()
        except StopAsyncIteration:
            return None

    @property
    def transport(self):
        return self._transport

    def write(self, obj):
        self._transport.write(self._packer.pack(obj))

    def write_eof(self):
        return self._transport.write_eof()

    def can_write_eof(self):
        return self._transport.can_write_eof()

    def close(self):
        return self._transport.close()

    def get_extra_info(self, name, default=None):
        return self._transport.get_extra_info(name, default)



import asyncio
import tempfile
import os.path

import asynctest
import msgpack

import aiomsgpack


class TestRead(asynctest.TestCase):

    async def test_basic(self):
        proto = aiomsgpack.MsgpackProtocol(None)
        proto.data_received(msgpack.packb(1))
        proto.data_received(msgpack.packb(2))

        self.assertEqual(await proto.read(), 1)
        self.assertEqual(await proto.read(), 2)

    async def test_eof(self):
        proto = aiomsgpack.MsgpackProtocol(None)
        proto.data_received(msgpack.packb(1))
        proto.eof_received()

        self.assertEqual(await proto.read(), 1)
        with self.assertRaises(StopAsyncIteration):
            await proto.__anext__()

        self.assertEqual(await proto.read(), None)

    async def test_partial(self):
        proto = aiomsgpack.MsgpackProtocol(None)
        msg = msgpack.packb(b'hello world')
        proto.data_received(msg[:5])  # Send partial message

        fut = asyncio.ensure_future(proto.__anext__())
        await asyncio.sleep(0.1)  # Yield to event loop
        self.assertFalse(fut.done())

        proto.data_received(msg[5:])  # Send remainder of message

        self.assertEqual(await fut, b'hello world')


class TestClientServer(asynctest.TestCase):

    async def test(self):

        with tempfile.TemporaryDirectory() as tmp_dir:
            path = os.path.join(tmp_dir, 'socket')

            server_proto_fut = asyncio.Future()

            async def server_connected_cb(proto):
                server_proto_fut.set_result(proto)

            async def client_connected_cb(proto):
                pass

            loop = asyncio.get_event_loop()

            server = await loop.create_unix_server(
                aiomsgpack.make_msgpack_protocol_factory(server_connected_cb),
                path
            )
            _, client_proto = await loop.create_unix_connection(
                aiomsgpack.make_msgpack_protocol_factory(client_connected_cb),
                path
            )
            server_proto = await server_proto_fut
            try:
                server_proto.write(b'hello')
                self.assertEqual(await client_proto.read(), b'hello')

                client_proto.write(b'hi')
                self.assertEqual(await server_proto.read(), b'hi')

            finally:
                client_proto.close()
                server_proto.close()
                server.close()
                await server.wait_closed()





#!/usr/bin/env python

from setuptools import setup

setup(
    setup_requires=['pbr'],
    pbr=True,
    py_modules=['aiomsgpack', ],
    include_package_data=True,
    test_suite='test_aiomsgpack',
)
